"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.signalingQueueManager = exports.SignalingQueueManager = void 0;
const dataSource_1 = require("../../database/dataSource");
const urlFromId_1 = require("../urlFromId");
const signalingQueue_1 = require("./signalingQueue");
const service_common_1 = require("@crosslab/service-common");
class SignalingQueueManager {
    constructor() {
        this.queueMap = new Map();
    }
    setOnCloseHandler(peerconnectionId, onClose) {
        const peerconnectionUrl = (0, urlFromId_1.peerconnectionUrlFromId)(peerconnectionId);
        const queues = this.queueMap.get(peerconnectionUrl);
        if (!queues)
            throw new service_common_1.MissingEntityError(`Peerconnection '${peerconnectionUrl}' does not have any associated signaling queues`, 404);
        queues.onClose = () => {
            onClose();
            this.queueMap.delete(peerconnectionId);
        };
    }
    async createSignalingQueues(peerconnectionId) {
        const peerconnectionUrl = (0, urlFromId_1.peerconnectionUrlFromId)(peerconnectionId);
        service_common_1.logger.log('info', `Trying to create signaling queues for peerconnection '${peerconnectionUrl}'`);
        // TODO: change to more meaningful error
        if (this.queueMap.has(peerconnectionUrl))
            throw new Error(`Peerconnection '${peerconnectionUrl}' already has signaling queues`);
        const peerconnectionModel = await dataSource_1.repositories.peerconnection.findOneOrFail({
            where: {
                uuid: peerconnectionId,
            },
        });
        if (peerconnectionModel.status !== 'new') {
            service_common_1.logger.log('info', `Status of peerconnection '${peerconnectionUrl}' is not 'new', '${peerconnectionModel.status}'`);
            return;
        }
        // create SignalingQueues
        const queueDeviceA = new signalingQueue_1.SignalingQueue(peerconnectionUrl, peerconnectionModel.deviceA.url);
        const queueDeviceB = new signalingQueue_1.SignalingQueue(peerconnectionUrl, peerconnectionModel.deviceB.url);
        // register onClose handlers
        queueDeviceA.onClose = () => {
            const queues = this.queueMap.get(peerconnectionUrl);
            if (queues?.deviceB.queue.state === 'peerconnection-closed' &&
                queues.onClose) {
                queues.onClose();
                this.queueMap.delete(peerconnectionUrl);
            }
        };
        queueDeviceB.onClose = () => {
            const queues = this.queueMap.get(peerconnectionUrl);
            if (queues?.deviceA.queue.state === 'peerconnection-closed' &&
                queues.onClose) {
                queues.onClose();
                this.queueMap.delete(peerconnectionUrl);
            }
        };
        // prepare createPeerconnection messages
        const common = {
            messageType: 'command',
            command: 'createPeerconnection',
            connectionType: peerconnectionModel.type,
            connectionUrl: peerconnectionUrl,
        };
        const createPeerConnectionMessageA = {
            ...common,
            services: peerconnectionModel.deviceA.config?.services ?? [],
            tiebreaker: false,
        };
        const createPeerConnectionMessageB = {
            ...common,
            services: peerconnectionModel.deviceB.config?.services ?? [],
            tiebreaker: true,
        };
        // add createPeerconnection messages
        queueDeviceA.add(createPeerConnectionMessageA);
        queueDeviceB.add(createPeerConnectionMessageB);
        this.queueMap.set(peerconnectionUrl, {
            deviceA: { url: peerconnectionModel.deviceA.url, queue: queueDeviceA },
            deviceB: { url: peerconnectionModel.deviceB.url, queue: queueDeviceB },
        });
        service_common_1.logger.log('info', `Successfully created signaling queues for peerconnection '${peerconnectionUrl}'`);
    }
    startSignalingQueues(peerconnectionId) {
        const peerconnectionUrl = (0, urlFromId_1.peerconnectionUrlFromId)(peerconnectionId);
        const queues = this.queueMap.get(peerconnectionUrl);
        if (!queues)
            throw new service_common_1.MissingEntityError(`Peerconnection '${peerconnectionUrl}' does not have any associated signaling queues`, 404);
        queues.deviceA.queue.start();
        queues.deviceB.queue.start();
    }
    stopSignalingQueues(peerconnectionId) {
        const peerconnectionUrl = (0, urlFromId_1.peerconnectionUrlFromId)(peerconnectionId);
        const queues = this.queueMap.get(peerconnectionUrl);
        if (!queues)
            throw new service_common_1.MissingEntityError(`Peerconnection '${peerconnectionUrl}' does not have any associated signaling queues`, 404);
        queues.deviceA.queue.stop();
        queues.deviceB.queue.stop();
    }
    addSignalingMessage(peerconnectionUrl, deviceUrl, signalingMessage) {
        const queues = this.queueMap.get(peerconnectionUrl);
        if (queues?.deviceA.url === deviceUrl)
            return queues.deviceA.queue.add(signalingMessage);
        if (queues?.deviceB.url === deviceUrl)
            return queues.deviceB.queue.add(signalingMessage);
        throw new service_common_1.MissingEntityError(`Peerconnection '${peerconnectionUrl}' does not have any associated signaling queue for device '${deviceUrl}'`, 404);
    }
    closeSignalingQueues(peerconnectionId) {
        const peerconnectionUrl = (0, urlFromId_1.peerconnectionUrlFromId)(peerconnectionId);
        const queues = this.queueMap.get(peerconnectionUrl);
        // TODO: change to more meaningful error
        if (!queues)
            throw new service_common_1.MissingEntityError(`Peerconnection '${peerconnectionUrl}' does not have any associated signaling queues`, 404);
        const closePeerconnectionMessage = {
            messageType: 'command',
            command: 'closePeerconnection',
            connectionUrl: peerconnectionUrl,
        };
        if (queues.deviceA.queue.state === 'new' && queues.deviceA.queue.onClose)
            queues.deviceA.queue.onClose();
        else
            queues.deviceA.queue.add(closePeerconnectionMessage);
        if (queues.deviceB.queue.state === 'new' && queues.deviceB.queue.onClose)
            queues.deviceB.queue.onClose();
        else
            queues.deviceB.queue.add(closePeerconnectionMessage);
    }
}
exports.SignalingQueueManager = SignalingQueueManager;
exports.signalingQueueManager = new SignalingQueueManager();
//# sourceMappingURL=signalingQueueManager.js.map