"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.postDevicesByDeviceId = void 0;
const dataSource_1 = require("../../../database/dataSource");
const globals_1 = require("../../../globals");
const callbacks_1 = require("../../../methods/callbacks");
const permission_1 = require("../../../methods/permission");
const urlFromId_1 = require("../../../methods/urlFromId");
const service_common_1 = require("@crosslab/service-common");
/**
 * This function implements the functionality for handling POST requests on /devices/{device_id} endpoint.
 * @param parameters The parameters of the request.
 * @param user The user submitting the request.
 * @throws {MissingEntityError} Thrown if device is not found in the database.
 * @throws {ImpossibleOperationError} Thrown if device is not instantiable.
 */
const postDevicesByDeviceId = async (parameters, user) => {
    service_common_1.logger.log('info', 'postDevicesByDeviceId called');
    const instantiableDeviceModel = await dataSource_1.repositories.device.findOneOrFail({
        where: { uuid: parameters.device_id },
    });
    if (!(0, permission_1.checkPermission)('instantiate', instantiableDeviceModel, user.JWT))
        throw new service_common_1.DeviceOwnershipError();
    if (instantiableDeviceModel.type !== 'cloud instantiable' &&
        instantiableDeviceModel.type !== 'edge instantiable')
        throw new service_common_1.ImpossibleOperationError(`Cannot create new instance of device '${(0, urlFromId_1.deviceUrlFromId)(instantiableDeviceModel.uuid)}' since it has type '${instantiableDeviceModel.type}'`, 400);
    const concreteDeviceModel = await dataSource_1.repositories.concreteDevice.create({
        ...(await dataSource_1.repositories.device.format(instantiableDeviceModel)),
        type: 'device',
    });
    concreteDeviceModel.owner = user.JWT?.url;
    await dataSource_1.repositories.device.save(concreteDeviceModel);
    if (parameters.changedUrl) {
        service_common_1.logger.log('info', `registering changed-callback for device '${(0, urlFromId_1.deviceUrlFromId)(concreteDeviceModel.uuid)}' to '${parameters.changedUrl}'`);
        const changedCallbackURLs = callbacks_1.changedCallbacks.get(concreteDeviceModel.uuid) ?? [];
        changedCallbackURLs.push(parameters.changedUrl);
        callbacks_1.changedCallbacks.set(concreteDeviceModel.uuid, changedCallbackURLs);
    }
    const instance = await dataSource_1.repositories.concreteDevice.format(concreteDeviceModel);
    const deviceToken = await globals_1.apiClient.createDeviceAuthenticationToken(instance.url); // TODO: error handling
    instantiableDeviceModel.instances ??= [];
    instantiableDeviceModel.instances.push(concreteDeviceModel);
    await dataSource_1.repositories.device.save(instantiableDeviceModel);
    service_common_1.logger.log('info', 'postDevicesByDeviceId succeeded');
    return {
        status: 201,
        body: {
            deviceToken,
            instance: instance,
        },
    };
};
exports.postDevicesByDeviceId = postDevicesByDeviceId;
//# sourceMappingURL=post.js.map