"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.postDevicesByDeviceIdSignaling = void 0;
const __1 = require("../..");
const dataSource_1 = require("../../../../database/dataSource");
const globals_1 = require("../../../../globals");
const urlFromId_1 = require("../../../../methods/urlFromId");
const service_common_1 = require("@crosslab/service-common");
/**
 * This function implements the functionality for handling POST requests on /devices/{device_id}/signaling endpoint.
 * @param parameters The parameters of the request.
 * @param body The body of the request.
 * @param _user The user submitting the request.
 * @throws {MissingEntityError} Thrown if device is not found in the database or if websocket for device is not found.
 * @throws {InvalidValueError} Thrown if type of device is not "device" or if device is not part of the peerconnection.
 */
const postDevicesByDeviceIdSignaling = async (parameters, body, _user) => {
    service_common_1.logger.log('info', 'postDevicesByDeviceIdSignaling called');
    // Get device
    const deviceModel = await dataSource_1.repositories.device.findOneOrFail({
        where: { uuid: parameters.device_id },
    });
    // Make sure device is a concrete device
    if (deviceModel.type !== 'device')
        throw new service_common_1.ImpossibleOperationError(`Cannot send signaling message to device with type '${deviceModel.type}'`, 400);
    // Retrieve peerconnection and make sure the device is taking part in it
    const peerconnection = await globals_1.apiClient.getPeerconnection(parameters.peerconnection_url);
    const deviceA = peerconnection.devices[0];
    const deviceB = peerconnection.devices[1];
    if (!(deviceA.url === (0, urlFromId_1.deviceUrlFromId)(deviceModel.uuid)) &&
        !(deviceB.url === (0, urlFromId_1.deviceUrlFromId)(deviceModel.uuid))) {
        throw new service_common_1.UnrelatedPeerconnectionError(`Device is not part of the peerconnection`, 400);
    }
    const webSocket = __1.connectedDevices.get(parameters.device_id);
    if (!webSocket)
        throw new service_common_1.MissingEntityError(`Could not find websocket connection for device ${parameters.device_id}`, 404);
    webSocket.send(JSON.stringify(body));
    service_common_1.logger.log('info', 'postDevicesByDeviceIdSignaling succeeded');
    return {
        status: 200,
    };
};
exports.postDevicesByDeviceIdSignaling = postDevicesByDeviceIdSignaling;
//# sourceMappingURL=post.js.map