"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.websocketHandling = exports.connectedDevices = void 0;
const dataSource_1 = require("../../../database/dataSource");
const types_1 = require("../../../generated/types");
const callbacks_1 = require("../../../methods/callbacks");
const urlFromId_1 = require("../../../methods/urlFromId");
const messageHandling_1 = require("./messageHandling");
const service_common_1 = require("@crosslab/service-common");
exports.connectedDevices = new Map();
class WebSocketConnectionError extends Error {
    constructor(message) {
        super(message);
        this.name = 'WebSocketConnectionError';
    }
}
/**
 * This function adds the /devices/ws endpoint, including its functionality, to an express application.
 * @param app The express application to add the /devices/ws endpoint to.
 */
function websocketHandling(app) {
    app.ws('/devices/websocket', (ws) => {
        // authenticate and start heartbeat
        ws.once('message', async (data) => {
            try {
                // device authentication and connection
                const message = JSON.parse(data.toString('utf8'));
                if (!((0, types_1.isMessage)(message) && (0, types_1.isAuthenticationMessage)(message))) {
                    service_common_1.logger.log('error', 'First received websocket message is not an authentication message');
                    return ws.close(1002, 'Received message is not an authentication message');
                }
                if (!message.token) {
                    service_common_1.logger.log('error', 'Authentication message does not contain a websocket token');
                    return ws.close(1002, 'Authentication message does not contain a websocket token');
                }
                const deviceModel = await dataSource_1.repositories.concreteDevice.findOne({
                    where: { token: message.token },
                });
                if (!deviceModel) {
                    service_common_1.logger.log('error', 'No device found with matching websocket token');
                    return ws.close(1002, 'No device found with matching websocket token');
                }
                ws.send(JSON.stringify({
                    messageType: 'authenticate',
                    authenticated: true,
                }));
                deviceModel.connected = true;
                exports.connectedDevices.set(deviceModel.uuid, ws);
                await dataSource_1.repositories.concreteDevice.save(deviceModel);
                await (0, callbacks_1.sendChangedCallback)(deviceModel);
                service_common_1.logger.log('info', `device '${(0, urlFromId_1.deviceUrlFromId)(deviceModel.uuid)}' connected`);
                // heartbeat implementation
                let isAlive = true;
                ws.on('pong', () => {
                    isAlive = true;
                    service_common_1.logger.log('info', `hearbeat received from device '${(0, urlFromId_1.deviceUrlFromId)(deviceModel.uuid)}'`);
                });
                const interval = setInterval(async function ping() {
                    try {
                        if (isAlive === false) {
                            service_common_1.logger.log('info', `Device '${(0, urlFromId_1.deviceUrlFromId)(deviceModel.uuid)}' did not answer hearbeat check, closing connection`);
                            deviceModel.connected = false;
                            await dataSource_1.repositories.concreteDevice.save(deviceModel);
                            await (0, callbacks_1.sendChangedCallback)(deviceModel);
                            exports.connectedDevices.delete(deviceModel.uuid);
                            clearInterval(interval);
                            return ws.terminate();
                        }
                        isAlive = false;
                        service_common_1.logger.log('info', `sending hearbeat to device '${(0, urlFromId_1.deviceUrlFromId)(deviceModel.uuid)}'`);
                        ws.ping();
                    }
                    catch (error) {
                        service_common_1.logger.log('error', `An error occurred during the heartbeat check of device '${(0, urlFromId_1.deviceUrlFromId)(deviceModel.uuid)}'`, { data: { error } });
                    }
                }, 30000);
                // close handler: stop heartbeat and disconnect device
                ws.on('close', async (code, reason) => {
                    clearInterval(interval);
                    deviceModel.connected = false;
                    await dataSource_1.repositories.concreteDevice.save(deviceModel);
                    await (0, callbacks_1.sendChangedCallback)(deviceModel);
                    exports.connectedDevices.delete(deviceModel.uuid);
                    service_common_1.logger.log('info', `websocket connection for device '${(0, urlFromId_1.deviceUrlFromId)(deviceModel.uuid)}' closed`);
                    if (code === 1002) {
                        service_common_1.logger.log('error', new WebSocketConnectionError(reason.toString('utf-8')));
                    }
                });
                // message handler: handle incoming messages from devices
                ws.on('message', async (rawData) => {
                    try {
                        const msg = JSON.parse(rawData.toString('utf-8'));
                        if (!(0, types_1.isMessage)(msg)) {
                            service_common_1.logger.log('error', `Received something that is not a message from device '${(0, urlFromId_1.deviceUrlFromId)(deviceModel.uuid)}', disconnecting`);
                            ws.close(1002, 'Malformed Message');
                            return;
                        }
                        await (0, messageHandling_1.handleDeviceMessage)(deviceModel, msg);
                    }
                    catch (error) {
                        service_common_1.logger.log('error', `An error occurred while handling an incoming message for device '${(0, urlFromId_1.deviceUrlFromId)(deviceModel.uuid)}'`, { data: { error } });
                    }
                });
            }
            catch (error) {
                service_common_1.logger.log('error', `Something went wrong during authentication or setup of a websocket connection`, { data: { error } });
                return ws.close(1002, 'Something went wrong during authentication or setup');
            }
        });
    });
}
exports.websocketHandling = websocketHandling;
//# sourceMappingURL=index.js.map