"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.patchPeerconnectionsByPeerconnectionIdDeviceStatus = void 0;
const dataSource_1 = require("../../../../database/dataSource");
const mutexManager_1 = require("../../../../methods/mutexManager");
const urlFromId_1 = require("../../../../methods/urlFromId");
const callbacks_1 = require("../../../callbacks");
const delete_1 = require("../delete");
const service_common_1 = require("@crosslab/service-common");
/**
 * This function implements the functionality for handling PATCH requests on
 * /peerconnections/{peerconnection_id}/device_status endpoint.
 * @param parameters The parameters of the request.
 * @param _user The user submitting the request.
 */
const patchPeerconnectionsByPeerconnectionIdDeviceStatus = async (parameters, body, _user) => {
    service_common_1.logger.log('info', 'patchPeerconnectionsByPeerconnectionIdDeviceStatus called', {
        data: {
            peerconnection: parameters.peerconnection_id,
            device: parameters.device_url,
            status: body.status,
        },
    });
    const release = await mutexManager_1.mutexManager.acquire(parameters.peerconnection_id);
    try {
        const peerconnectionModel = await dataSource_1.repositories.peerconnection.findOneOrFail({
            where: { uuid: parameters.peerconnection_id },
        });
        if (peerconnectionModel.deviceA.url !== parameters.device_url &&
            peerconnectionModel.deviceB.url !== parameters.device_url) {
            throw new service_common_1.UnrelatedPeerconnectionError(`Device '${parameters.device_url}' is not taking part in peerconnection '${(0, urlFromId_1.peerconnectionUrlFromId)(peerconnectionModel.uuid)}'`, 400);
        }
        if (peerconnectionModel.deviceA.url === parameters.device_url)
            peerconnectionModel.deviceA.status = body.status;
        if (peerconnectionModel.deviceB.url === parameters.device_url)
            peerconnectionModel.deviceB.status = body.status;
        const oldStatus = peerconnectionModel.status;
        if (peerconnectionModel.status === 'closed' ||
            peerconnectionModel.deviceA.status === 'closed' ||
            peerconnectionModel.deviceB.status === 'closed') {
            peerconnectionModel.status = 'closed';
        }
        else if (peerconnectionModel.deviceA.status === 'failed' ||
            peerconnectionModel.deviceB.status === 'failed') {
            peerconnectionModel.status = 'failed';
        }
        else if (peerconnectionModel.deviceA.status === 'disconnected' ||
            peerconnectionModel.deviceB.status === 'disconnected') {
            peerconnectionModel.status = 'disconnected';
        }
        else if (peerconnectionModel.deviceA.status === 'connecting' ||
            peerconnectionModel.deviceB.status === 'connecting') {
            peerconnectionModel.status = 'connecting';
        }
        else if (peerconnectionModel.deviceA.status === 'connected' &&
            peerconnectionModel.deviceB.status === 'connected') {
            peerconnectionModel.status = 'connected';
        }
        else {
            peerconnectionModel.status = 'new';
        }
        if (peerconnectionModel.status !== oldStatus) {
            await (0, callbacks_1.sendStatusChangedCallback)(peerconnectionModel);
            if (peerconnectionModel.status === 'closed')
                await (0, callbacks_1.sendClosedCallback)(peerconnectionModel);
        }
        service_common_1.logger.log('info', 'DEVICE STATUS INFO', {
            data: {
                peerconnection: peerconnectionModel.uuid,
                statusDeviceA: peerconnectionModel.deviceA.status,
                statusDeviceB: peerconnectionModel.deviceB.status,
                toBeDeleted: delete_1.deleteOnClose.has(peerconnectionModel.uuid),
            },
        });
        if (peerconnectionModel.deviceA.status === 'closed' &&
            peerconnectionModel.deviceB.status === 'closed' &&
            delete_1.deleteOnClose.has(peerconnectionModel.uuid)) {
            await dataSource_1.repositories.peerconnection.remove(peerconnectionModel);
        }
        else {
            await dataSource_1.repositories.peerconnection.save(peerconnectionModel);
        }
        service_common_1.logger.log('info', 'patchPeerconnectionsByPeerconnectionIdDeviceStatus succeeded');
    }
    finally {
        release();
    }
    return {
        status: 204,
    };
};
exports.patchPeerconnectionsByPeerconnectionIdDeviceStatus = patchPeerconnectionsByPeerconnectionIdDeviceStatus;
//# sourceMappingURL=patch.js.map