"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.postPeerconnections = void 0;
const dataSource_1 = require("../../database/dataSource");
const globals_1 = require("../../globals");
const callbacks_1 = require("../../methods/callbacks");
const signalingQueueManager_1 = require("../../methods/signaling/signalingQueueManager");
const urlFromId_1 = require("../../methods/urlFromId");
const service_common_1 = require("@crosslab/service-common");
/**
 * This function implements the functionality for handling POST requests on /peerconnections endpoint.
 * @param parameters The parameters of the request.
 * @param body The body of the request.
 * @param _user The user submitting the request.
 */
const postPeerconnections = async (parameters, body, _user) => {
    service_common_1.logger.log('info', 'postPeerconnections called');
    const deviceA = await globals_1.apiClient.getDevice(body.devices[0].url);
    const deviceB = await globals_1.apiClient.getDevice(body.devices[1].url);
    if (deviceA.type !== 'device' || deviceB.type !== 'device') {
        throw new service_common_1.InvalidValueError(`Cannot establish a peerconnection between devices of type '${deviceA.type}' and '${deviceB.type}'`, 400);
    }
    const peerconnectionModel = await dataSource_1.repositories.peerconnection.create(body);
    await dataSource_1.repositories.peerconnection.save(peerconnectionModel);
    await signalingQueueManager_1.signalingQueueManager.createSignalingQueues(peerconnectionModel.uuid);
    if (deviceA.connected && deviceB.connected) {
        // peerconnection can be started directly
        signalingQueueManager_1.signalingQueueManager.startSignalingQueues(peerconnectionModel.uuid);
    }
    else {
        // need to wait for devices to connect
        // register changed callbacks for devices to get notified when they connect
        const n_deviceA = await globals_1.apiClient.updateDevice(deviceA.url, { type: 'device' }, { changedUrl: callbacks_1.callbackUrl });
        const n_deviceB = await globals_1.apiClient.updateDevice(deviceB.url, { type: 'device' }, { changedUrl: callbacks_1.callbackUrl });
        // check that devices still have the correct type
        if (n_deviceA.type !== 'device' || n_deviceB.type !== 'device') {
            throw new service_common_1.InvalidValueError(`Cannot establish a peerconnection between devices of type '${deviceA.type}' and '${deviceB.type}'`, 400);
        }
        // set timeout for checking if devices are connected
        globals_1.timeoutMap.set(peerconnectionModel.uuid, setTimeout(async () => {
            try {
                service_common_1.logger.log('info', 'devices did not connect');
                peerconnectionModel.status = 'failed';
                await dataSource_1.repositories.peerconnection.save(peerconnectionModel);
                await (0, callbacks_1.sendStatusChangedCallback)(peerconnectionModel);
            }
            catch (error) {
                service_common_1.logger.log('error', `Something went wrong while trying to set status of peerconnection '${(0, urlFromId_1.peerconnectionUrlFromId)(peerconnectionModel.uuid)}' to 'failed'`, { data: { error } });
            }
        }, 30000));
    }
    if (parameters.closedUrl) {
        service_common_1.logger.log('info', `postPeerconnections: registering closed-callback for '${parameters.closedUrl}'`);
        const closedCallbackURLs = callbacks_1.closedCallbacks.get(peerconnectionModel.uuid) ?? [];
        closedCallbackURLs.push(parameters.closedUrl);
        callbacks_1.closedCallbacks.set(peerconnectionModel.uuid, closedCallbackURLs);
    }
    if (parameters.statusChangedUrl) {
        service_common_1.logger.log('info', `postPeerconnections: registering status-changed-callback for '${parameters.statusChangedUrl}'`);
        const statusChangedCallbackURLs = callbacks_1.statusChangedCallbacks.get(peerconnectionModel.uuid) ?? [];
        statusChangedCallbackURLs.push(parameters.statusChangedUrl);
        callbacks_1.statusChangedCallbacks.set(peerconnectionModel.uuid, statusChangedCallbackURLs);
    }
    service_common_1.logger.log('info', 'postPeerconnections succeeded');
    return {
        status: peerconnectionModel.status === 'connected' ? 201 : 202,
        body: await dataSource_1.repositories.peerconnection.format(peerconnectionModel),
    };
};
exports.postPeerconnections = postPeerconnections;
//# sourceMappingURL=post.js.map